@@ -12,7 +12,6 @@ import insightconnect_plugin_runtime.dispatcher
 import insightconnect_plugin_runtime.exceptions
 import certifi
 import os
-import logging
 
 Plugin = insightconnect_plugin_runtime.plugin.Plugin
 Action = insightconnect_plugin_runtime.action.Action
@@ -56,7 +55,3 @@ def where():
 
 # and here's the monkey-patch itself.
 certifi.where = where
-
-root_logger = logging.getLogger()
-root_logger.setLevel("DEBUG")
-root_logger.addHandler(logging.StreamHandler())
