@@ -2,8 +2,11 @@ import json
 import subprocess
 import yaml
 import os
+import pkg_resources
 import signal
 from flask import jsonify, request, abort, make_response, Blueprint
+import structlog
+
 from insightconnect_plugin_runtime.exceptions import (
     ClientException,
     ServerException,
@@ -22,7 +25,7 @@ from insightconnect_plugin_runtime.api.schemas import (
 class Endpoints:
     def __init__(self, logger, plugin, spec, debug, workers, threads, master_pid):
         self.plugin = plugin
-        self.logger = logger
+        self.logger = structlog.get_logger("plugin")
         self.spec = spec
         self.debug = debug
         self.workers = workers
@@ -263,7 +266,11 @@ class Endpoints:
             response = Endpoints.get_plugin_info(plugin_spec_json, plugin_info_fields)
             # Add workers and threads
             response.update(
-                {"number_of_workers": self.workers, "threads": self.threads}
+                {
+                    "number_of_workers": self.workers,
+                    "threads": self.threads,
+                    "sdk_version": self.get_plugin_sdk_version(),
+                }
             )
             return jsonify(PluginInfoSchema().dump(response))
 
@@ -485,7 +492,7 @@ class Endpoints:
             Adds a worker (another process)
             :return: Json Response
             """
-            r = {}
+            response = {}
 
             # Linux signal examples here:
             # https://docs.gunicorn.org/en/stable/signals.html#master-process
@@ -493,13 +500,13 @@ class Endpoints:
                 self.logger.info("Adding a worker")
                 self.logger.info("Current process is: %s" % self.master_pid)
                 os.kill(self.master_pid, signal.SIGTTIN)
-            except Exception as e:
-                r.status_code = 500
-                r.error = e
-                return jsonify(r)
+            except Exception as error:
+                response.status_code = 500
+                response.error = error
+                return jsonify(response)
 
-            r["num_workers"] = Endpoints._number_of_workers()
-            return jsonify(r)
+            response["num_workers"] = Endpoints._number_of_workers()
+            return jsonify(response)
 
         @v1.route("/workers/remove", methods=["POST"])
         def remove_worker():
@@ -510,7 +517,7 @@ class Endpoints:
             :return: Json Response
             """
 
-            r = {}
+            response = {}
 
             # Linux signal examples here:
             # https://docs.gunicorn.org/en/stable/signals.html#master-process
@@ -518,18 +525,18 @@ class Endpoints:
                 self.logger.info("Removing a worker")
                 self.logger.info("Current process is: %s" % self.master_pid)
                 os.kill(self.master_pid, signal.SIGTTOU)
-            except Exception as e:
-                r = {}
-                r.status_code = 500
-                r.error = e
-                return jsonify(r)
+            except Exception as error:
+                response = {}
+                response.status_code = 500
+                response.error = error
+                return jsonify(response)
 
-            return jsonify(r)  # Flask or Gunicorn expect a return
+            return jsonify(response)  # Flask or Gunicorn expect a return
 
         @v1.route("/workers", methods=["GET"])
         def num_workers():
-            r = {"num_workers": Endpoints._number_of_workers()}
-            return jsonify(r)
+            response = {"num_workers": Endpoints._number_of_workers()}
+            return jsonify(response)
 
         @v1.route("/connection", methods=["GET"])
         def connection():
@@ -575,19 +582,22 @@ class Endpoints:
             Endpoints.validate_action_trigger_task_empty_input(input_message)
 
             try:
-                output = self.plugin.handle_step(input_message, is_debug=self.debug, is_test=True)
+                output = self.plugin.handle_step(
+                    input_message, is_debug=self.debug, is_test=True
+                )
                 if output.get("body", {}).get("output") is None:
                     status_code = 204
-            except LoggedException as e:
-                wrapped_exception = e.ex
+            except LoggedException as error:
+                wrapped_exception = error.ex
                 self.logger.exception(wrapped_exception)
 
-                output = e.output
+                output = error.output
                 status_code = Endpoints.handle_wrapped_exception(wrapped_exception)
             finally:
-                r = jsonify(output)
-                r.status_code = status_code
-                return r
+                response = jsonify(output)
+                response.status_code = status_code
+                return response
+
         blueprints = [legacy, v1]
         return blueprints
 
@@ -630,20 +640,20 @@ class Endpoints:
         actions_triggers_tasks = plugin_spec_json.get(p_type)
         if actions_triggers_tasks is None or actions_triggers_tasks.get(p_name) is None:
             msg = f"{p_type[:-1].capitalize()} {p_name} does not exist"
-            resp = make_response(jsonify({"error": msg}), 400)
-            abort(resp)
+            response = make_response(jsonify({"error": msg}), 400)
+            abort(response)
         return actions_triggers_tasks.get(p_name)
 
     @staticmethod
     def load_file_json_format(filename):
-        with open(filename, "r") as p_spec:
-            return yaml.safe_load(p_spec.read())
+        with open(filename, "r") as plugin_spec:
+            return yaml.safe_load(plugin_spec.read())
 
     @staticmethod
     def validate_action_trigger_task_empty_input(input_message):
         if not input_message:
-            resp = make_response(jsonify({"error": "Empty input provided"}), 400)
-            abort(resp)
+            response = make_response(jsonify({"error": "Empty input provided"}), 400)
+            abort(response)
 
     @staticmethod
     def validate_action_trigger_task_name(input_message, name, p_type):
@@ -660,6 +670,15 @@ class Endpoints:
             plugin_info.update({field: plugin_spec_json.get(field)})
         return plugin_info
 
+    def get_plugin_sdk_version(self):
+        try:
+            version = pkg_resources.require("insightconnect-plugin-runtime")[0].version
+        except Exception:
+            self.logger.warn("Unable to get SDK version")
+            version = "0.0.0"
+
+        return version
+
     def run_action_trigger_task(self, input_message, test=False):
         status_code = 200
         output = None
@@ -667,15 +686,17 @@ class Endpoints:
             output = self.plugin.handle_step(
                 input_message, is_debug=self.debug, is_test=test
             )
-        except LoggedException as e:
-            wrapped_exception = e.ex
+        except LoggedException as error:
+            wrapped_exception = error.ex
             self.logger.exception(wrapped_exception)
 
-            output = e.output
+            output = error.output
             if isinstance(wrapped_exception, ClientException):
                 status_code = 400
-            elif isinstance(wrapped_exception, PluginException) and \
-                    wrapped_exception.preset is PluginException.Preset.BAD_REQUEST:
+            elif (
+                isinstance(wrapped_exception, PluginException)
+                and wrapped_exception.preset is PluginException.Preset.BAD_REQUEST
+            ):
                 status_code = 400
             elif isinstance(wrapped_exception, ServerException):
                 # I'm unsure about this
@@ -684,16 +705,18 @@ class Endpoints:
                 status_code = 500
         finally:
             self.logger.debug("Request output: %s", output)
-            r = jsonify(output)
-            r.status_code = status_code
-            return r
+            response = jsonify(output)
+            response.status_code = status_code
+            return response
 
     @staticmethod
     def handle_wrapped_exception(wrapped_exception):
         if isinstance(wrapped_exception, (ConnectionTestException, ClientException)):
             return 400
-        elif isinstance(wrapped_exception, PluginException) and \
-                wrapped_exception.preset is PluginException.Preset.BAD_REQUEST:
+        elif (
+            isinstance(wrapped_exception, PluginException)
+            and wrapped_exception.preset is PluginException.Preset.BAD_REQUEST
+        ):
             return 400
         elif isinstance(wrapped_exception, ServerException):
             return 500
