@@ -1,23 +1,36 @@
 import os
 import sys
 import json
-import gunicorn.app.base
-from flask import Flask
-from gunicorn.arbiter import Arbiter
+import logging
+
 from apispec import APISpec
 from apispec.ext.marshmallow import MarshmallowPlugin
 from apispec_webframeworks.flask import FlaskPlugin
+
+from flask import Flask, request_started, request
+import gunicorn.app.base
+from gunicorn.arbiter import Arbiter
+
+import structlog
+from pythonjsonlogger.jsonlogger import JsonFormatter
+
 from insightconnect_plugin_runtime.api.schemas import (
     PluginInfoSchema,
     ActionTriggerOutputBodySchema,
     ActionTriggerOutputSchema,
+    TaskOutputBodySchema,
+    TaskOutputSchema,
     ActionTriggerInputBodySchema,
     ActionTriggerInputSchema,
+    TaskInputBodySchema,
+    TaskInputSchema,
     ActionTriggerDetailsSchema,
+    TaskDetailsSchema,
     ConnectionDetailsSchema,
     ConnectionTestSchema,
 )
 from insightconnect_plugin_runtime.api.endpoints import Endpoints
+from insightconnect_plugin_runtime.util import is_running_in_cloud
 
 API_TITLE = "InsightConnect Plugin Runtime API"
 API_VERSION = "1.0"
@@ -39,8 +52,16 @@ class PluginServer(gunicorn.app.base.BaseApplication):
 
     """
 
-    def __init__(self, plugin, port=10001, workers=1, threads=4, debug=False,
-                 worker_class='sync', worker_connections=200):
+    def __init__(
+        self,
+        plugin,
+        port=10001,
+        workers=1,
+        threads=4,
+        debug=False,
+        worker_class="sync",
+        worker_connections=200,
+    ):
 
         if os.environ.get("GUNICORN_CONFIG_FILE"):
             with open(os.environ.get("GUNICORN_CONFIG_FILE")) as gf:
@@ -52,15 +73,18 @@ class PluginServer(gunicorn.app.base.BaseApplication):
                 "worker_class": worker_class,
                 "loglevel": "debug" if debug else "info",
             }
-            if worker_class == 'gevent':
-                self.gunicorn_config['worker_connections'] = worker_connections
+            if worker_class == "gevent":
+                self.gunicorn_config["worker_connections"] = worker_connections
             else:
-                self.gunicorn_config['threads'] = threads
+                self.gunicorn_config["threads"] = threads
 
         super(PluginServer, self).__init__()
         self.plugin = plugin
         self.arbiter = Arbiter(self)
-        self.logger = self.arbiter.log
+
+        self.configure_structlog_instance(is_debug=debug)
+        self.logger = structlog.get_logger("plugin")
+
         self.debug = debug
         # Create an APISpec
         self.spec = APISpec(
@@ -73,6 +97,35 @@ class PluginServer(gunicorn.app.base.BaseApplication):
         self.threads = threads
         self.app, self.blueprints = self.create_flask_app()
 
+    @staticmethod
+    def configure_structlog_instance(is_debug: bool) -> None:
+        structlog.configure(
+            processors=[
+                structlog.stdlib.merge_contextvars,  # Preserves contextvars, must come first in processors list
+                structlog.stdlib.filter_by_level,
+                structlog.stdlib.add_log_level,
+                structlog.stdlib.PositionalArgumentsFormatter(),
+                structlog.processors.StackInfoRenderer(),
+                structlog.processors.UnicodeDecoder(),
+                structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S"),
+                structlog.stdlib.render_to_log_kwargs,
+            ],
+            context_class=dict,
+            logger_factory=structlog.stdlib.LoggerFactory(),
+            wrapper_class=structlog.stdlib.BoundLogger,
+            cache_logger_on_first_use=True,
+        )
+        logger = logging.getLogger("plugin")
+        logger.setLevel(logging.DEBUG if is_debug else logging.INFO)
+
+        console_handler = logging.StreamHandler()
+        if is_running_in_cloud():
+            console_handler.setFormatter(
+                JsonFormatter()
+            )  # Only log in JSON if running in cloud
+
+        logger.addHandler(console_handler)
+
     def init(self, parser, opts, args):
         pass
 
@@ -92,6 +145,11 @@ class PluginServer(gunicorn.app.base.BaseApplication):
 
     def create_flask_app(self):
         app = Flask(__name__)
+
+        # Add structured logging middlware - requires 'blinker' library
+        # See https://flask.palletsprojects.com/en/2.2.x/signals/
+        request_started.connect(self.bind_request_details, app)
+
         endpoints = Endpoints(
             self.logger,
             self.plugin,
@@ -106,7 +164,7 @@ class PluginServer(gunicorn.app.base.BaseApplication):
         return app, blueprints
 
     def register_api_spec(self):
-        """ Register all swagger schema definitions and path objects """
+        """Register all swagger schema definitions and path objects"""
         self.spec.components.schema("PluginInfo", schema=PluginInfoSchema)
         self.spec.components.schema(
             "ActionTriggerOutputBody", schema=ActionTriggerOutputBodySchema
@@ -114,30 +172,37 @@ class PluginServer(gunicorn.app.base.BaseApplication):
         self.spec.components.schema(
             "ActionTriggerOutput", schema=ActionTriggerOutputSchema
         )
+        self.spec.components.schema("TaskOutputBody", schema=TaskOutputBodySchema)
+        self.spec.components.schema("TaskOutput", schema=TaskOutputSchema)
         self.spec.components.schema(
             "ActionTriggerInputBody", schema=ActionTriggerInputBodySchema
         )
         self.spec.components.schema(
             "ActionTriggerInput", schema=ActionTriggerInputSchema
         )
+        self.spec.components.schema("TaskInputBody", schema=TaskInputBodySchema)
+        self.spec.components.schema("TaskInput", schema=TaskInputSchema)
         self.spec.components.schema(
             "ActionTriggerDetails", schema=ActionTriggerDetailsSchema
         )
-        self.spec.components.schema("ConnectionDetails",
-                                    schema=ConnectionDetailsSchema)
-        self.spec.components.schema("ConnectionTestOutput",
-                                    schema=ConnectionTestSchema)
+        self.spec.components.schema("ConnectionDetails", schema=ConnectionDetailsSchema)
+        self.spec.components.schema("ConnectionTestOutput", schema=ConnectionTestSchema)
+        self.spec.components.schema("TaskDetails", schema=TaskDetailsSchema)
         self.spec.path(view=self.app.view_functions["v1.api_spec"])
         self.spec.path(view=self.app.view_functions["v1.plugin_info"])
         self.spec.path(view=self.app.view_functions["v1.plugin_spec"])
         self.spec.path(view=self.app.view_functions["v1.actions"])
         self.spec.path(view=self.app.view_functions["v1.triggers"])
+        self.spec.path(view=self.app.view_functions["v1.tasks"])
         self.spec.path(view=self.app.view_functions["v1.status"])
         self.spec.path(view=self.app.view_functions["v1.action_run"])
+        self.spec.path(view=self.app.view_functions["v1.task_run"])
         self.spec.path(view=self.app.view_functions["v1.action_test"])
         self.spec.path(view=self.app.view_functions["v1.trigger_test"])
+        self.spec.path(view=self.app.view_functions["v1.task_test"])
         self.spec.path(view=self.app.view_functions["v1.action_details"])
         self.spec.path(view=self.app.view_functions["v1.trigger_details"])
+        self.spec.path(view=self.app.view_functions["v1.task_details"])
         self.spec.path(view=self.app.view_functions["v1.connection"])
         self.spec.path(view=self.app.view_functions["v1.connection_test"])
 
@@ -148,8 +213,29 @@ class PluginServer(gunicorn.app.base.BaseApplication):
                 blueprint, url_prefix=VERSION_MAPPING[blueprint.name]
             )
 
+    @staticmethod
+    def bind_request_details(sender: Flask, **extras) -> None:
+        """
+        Middleware for binding request ID headers to structlog context
+        @param sender: Flask instance
+        @param extras: kwargs
+        @return: None
+        """
+        request_id = "X-REQUEST-ID"
+
+        structlog.contextvars.clear_contextvars()
+        if is_running_in_cloud():
+            try:
+                structlog.contextvars.bind_contextvars(
+                    request_id=request.headers[request_id]
+                )
+            except KeyError as error:
+                raise Exception(
+                    f"Failed to get {request_id} from headers while plugin is running in cloud."
+                ) from error
+
     def start(self):
-        """ start server """
+        """start server"""
         with self.app.app_context():
             try:
                 self.register_blueprint()
